@@ -28,19 +28,17 @@ import os
 import socket
 import sys
 import time
-import tarfile
 from collections import deque
-from six.moves.configparser import RawConfigParser
+from configparser import RawConfigParser
 from threading import Lock, Thread, Event
 import hashlib
-import six
-from six.moves.urllib.parse import urlparse, urlunparse
+from urllib.parse import urlparse, urlunparse
 import subprocess
 from contextlib import suppress
-import pyinotify
+
+import tarfile
 from zmq import LINGER, POLLIN, REQ, Poller
 import bz2
-
 from posttroll import get_context
 from posttroll.message import Message, MessageError
 from posttroll.publisher import NoisyPublisher
@@ -73,7 +71,6 @@ BUNZIP_BLOCK_SIZE = 1024
 LISTENER_CHECK_INTERVAL = 1
 
 
-# Config management
 def read_config(filename):
     """Read the config file called *filename*."""
     cp_ = RawConfigParser()
@@ -83,67 +80,89 @@ def read_config(filename):
 
     for section in cp_.sections():
         res[section] = dict(cp_.items(section))
-        res[section].setdefault("delete", False)
-        if res[section]["delete"] in ["", "False", "false", "0", "off"]:
-            res[section]["delete"] = False
-        if res[section]["delete"] in ["True", "true", "on", "1"]:
-            res[section]["delete"] = True
-        res[section].setdefault("working_directory", None)
-        res[section].setdefault("compression", False)
-        res[section].setdefault("xritdecompressor", None)
-        res[section].setdefault("heartbeat", True)
-        res[section].setdefault("req_timeout", DEFAULT_REQ_TIMEOUT)
-        res[section].setdefault("transfer_req_timeout", 10 * DEFAULT_REQ_TIMEOUT)
-        res[section].setdefault("nameservers", None)
-        if res[section]["heartbeat"] in ["", "False", "false", "0", "off"]:
-            res[section]["heartbeat"] = False
-
-        if "providers" not in res[section]:
-            LOGGER.warning("Incomplete section %s: add an 'providers' item.",
-                           section)
-            LOGGER.info("Ignoring section %s: incomplete.",
-                        section)
-            del res[section]
+        _set_config_defaults(res[section])
+        _parse_boolean_config_items(res[section])
+        if not _check_provider_config(res, section):
             continue
-        else:
-            res[section]["providers"] = [
-                "tcp://" + item.split('/', 1)[0] for item in res[section]["providers"].split()
-            ]
-
-        if "destination" not in res[section]:
-            LOGGER.warning("Incomplete section %s: add an 'destination' item.",
-                           section)
-            LOGGER.info("Ignoring section %s: incomplete.", section)
-            del res[section]
+        if not _check_destination(res, section):
             continue
-
-        if "topic" in res[section]:
-            try:
-                res[section]["publish_port"] = int(res[section][
-                    "publish_port"])
-            except (KeyError, ValueError):
-                res[section]["publish_port"] = 0
-        elif not res[section]["heartbeat"]:
-            # We have no topics and therefor no subscriber (if you want to
-            # subscribe everything, then explicit specify an empty topic).
-            LOGGER.warning("Incomplete section %s: add an 'topic' "
-                           "item or enable heartbeat.", section)
-            LOGGER.info("Ignoring section %s: incomplete.", section)
-            del res[section]
+        if not _check_subscribing(res, section):
             continue
 
     return res
 
 
+def _set_config_defaults(conf):
+    conf.setdefault("delete", False)
+    conf.setdefault("working_directory", None)
+    conf.setdefault("compression", False)
+    conf.setdefault("xritdecompressor", None)
+    conf.setdefault("heartbeat", True)
+    conf.setdefault("req_timeout", DEFAULT_REQ_TIMEOUT)
+    conf.setdefault("transfer_req_timeout", 10 * DEFAULT_REQ_TIMEOUT)
+    conf.setdefault("nameservers", None)
+
+
+def _parse_boolean_config_items(conf):
+    if conf["delete"] in ["", "False", "false", "0", "off"]:
+        conf["delete"] = False
+    if conf["delete"] in ["True", "true", "on", "1"]:
+        conf["delete"] = True
+    if conf["heartbeat"] in ["", "False", "false", "0", "off"]:
+        conf["heartbeat"] = False
+
+
+def _check_provider_config(conf, section):
+    if "providers" not in conf[section]:
+        LOGGER.warning("Incomplete section %s: add an 'providers' item.",
+                       section)
+        LOGGER.info("Ignoring section %s: incomplete.",
+                    section)
+        del conf[section]
+        return False
+
+    conf[section]["providers"] = [
+        "tcp://" + item.split('/', 1)[0] for item in conf[section]["providers"].split()
+    ]
+    return True
+
+
+def _check_subscribing(res, section):
+    if "topic" in res[section]:
+        try:
+            res[section]["publish_port"] = int(res[section][
+                "publish_port"])
+        except (KeyError, ValueError):
+            res[section]["publish_port"] = 0
+    elif not res[section]["heartbeat"]:
+        # We have no topics and therefor no subscriber (if you want to
+        # subscribe everything, then explicit specify an empty topic).
+        LOGGER.warning("Incomplete section %s: add an 'topic' "
+                       "item or enable heartbeat.", section)
+        LOGGER.info("Ignoring section %s: incomplete.", section)
+        del res[section]
+        return False
+    return True
+
+
+def _check_destination(res, section):
+    if "destination" not in res[section]:
+        LOGGER.warning("Incomplete section %s: add an 'destination' item.",
+                       section)
+        LOGGER.info("Ignoring section %s: incomplete.", section)
+        del res[section]
+        return False
+    return True
+
+
 class Listener(Thread):
     """PyTroll listener class for reading messages for Trollduction."""
 
-    def __init__(self, address, topics, callback, *args, die_event=None, **kwargs):
+    def __init__(self, address, topics, *args, die_event=None, **kwargs):
         """Init Listener object."""
         super(Listener, self).__init__()
 
         self.topics = topics
-        self.callback = callback
         self.subscriber = None
         self.address = address
         self.running = False
@@ -157,7 +176,7 @@ class Listener(Thread):
     def restart(self):
         """Restart the listener, returns a new running instance."""
         self.stop()
-        new_listener = self.__class__(self.address, self.topics, self.callback, *self.cargs,
+        new_listener = self.__class__(self.address, self.topics, *self.cargs,
                                       die_event=self.die_event, **self.ckwargs)
         new_listener.death_count = self.death_count + 1
         new_listener.start()
@@ -176,78 +195,67 @@ class Listener(Thread):
         """Run listener."""
         try:
             with heartbeat_monitor.Monitor(self.restart_event, **self.ckwargs) as beat_monitor:
-
                 self.running = True
-
                 while self.running:
-                    # Loop for restart.
                     LOGGER.debug("Starting listener %s", str(self.address))
                     self.create_subscriber()
-
-                    for msg in self.subscriber(timeout=1):
-                        if not self.running:
-                            break
-
-                        # If the heartbeat didn't arrive, restart the subscriber
-                        if self.restart_event.is_set():
-                            LOGGER.warning("Missing a heartbeat, restarting the subscriber to %s.",
-                                           str(self.subscriber.addresses))
-                            self.restart_event.clear()
-                            self.stop()
-                            self.running = True
-                            break
-
-                        if msg is None:
-                            continue
-
-                        LOGGER.debug("Receiving (SUB) %s", str(msg))
-
-                        beat_monitor(msg)
-                        if msg.type == "beat":
-                            self.death_count = 0
-                            continue
-
-                        # Handle public "push" messages as a hot spare client
-                        if msg.type == "push":
-                            # TODO: these need to be checked and acted if
-                            # the transfers are not finished on primary
-                            # client and are not cleared
-                            LOGGER.debug("Primary client published 'push'")
-                            add_to_ongoing(msg)
-                            continue
-
-                        # Handle public "ack" messages as a hot spare client
-                        if msg.type == "ack":
-                            LOGGER.debug("Primary client finished transfer")
-                            _ = add_to_file_cache(msg)
-                            _ = clean_ongoing_transfer(get_msg_uid(msg))
-                            continue
-
-                        # Ignore "file" messages which don't have a request address
-                        if msg.type == "file" and "request_address" not in msg.data:
-                            LOGGER.debug("Ignoring 'file' message from primary client.")
-                            add_to_ongoing(msg)
-                            _ = add_to_file_cache(msg)
-                            _ = clean_ongoing_transfer(get_msg_uid(msg))
-                            continue
-
-                        # If this is a hot spare client, wait for a while
-                        # for a public "push" message which will update
-                        # the ongoing transfers before starting processing here
-                        delay = self.ckwargs.get("processing_delay", False)
-                        if delay:
-                            add_timer(float(delay), self.callback, msg, *self.cargs,
-                                      **self.ckwargs)
-                        else:
-                            self.callback(msg, *self.cargs, **self.ckwargs)
-
-                    LOGGER.debug("Exiting listener %s", str(self.address))
+                    self._get_messages(beat_monitor)
         except Exception as err:
             LOGGER.exception("Listener died.")
             self.cause_of_death = err
             with suppress(AttributeError):
                 self.die_event.set()
 
+    def _get_messages(self, beat_monitor):
+        for msg in self.subscriber(timeout=1):
+            if not self.running:
+                break
+            if not self._check_heartbeat():
+                break
+            if msg is None:
+                continue
+
+            LOGGER.debug("Receiving (SUB) %s", str(msg))
+
+            beat_monitor(msg)
+
+            if self._is_message_already_handled(msg):
+                continue
+
+            self._process_message(msg)
+
+        LOGGER.debug("Exiting listener %s", str(self.address))
+
+    def _check_heartbeat(self):
+        if self.restart_event.is_set():
+            LOGGER.warning("Missing a heartbeat, restarting the subscriber to %s.",
+                           str(self.subscriber.addresses))
+            self.restart_event.clear()
+            self.stop()
+            self.running = True
+            return False
+        return True
+
+    def _is_message_already_handled(self, msg):
+        return (self._handle_beat_message(msg) or _handle_push_message(msg) or
+                _handle_ack_message(msg) or _handle_message_from_another_client(msg))
+
+    def _handle_beat_message(self, msg):
+        if msg.type == "beat":
+            self.death_count = 0
+            return True
+        return False
+
+    def _process_message(self, msg):
+        delay = self.ckwargs.get("processing_delay", False)
+        if delay:
+            # If this is a hot spare client, wait for a while
+            # for a public "push" message which will update
+            # the ongoing transfers before starting processing here
+            add_request_push_timer(float(delay), msg, *self.cargs, **self.ckwargs)
+        else:
+            request_push(msg, *self.cargs, **self.ckwargs)
+
     def stop(self):
         """Stop subscriber and delete the instance."""
         self.running = False
@@ -257,6 +265,36 @@ class Listener(Thread):
             self.subscriber = None
 
 
+def _handle_push_message(msg):
+    if msg.type == "push":
+        # TODO: these need to be checked and acted if
+        # the transfers are not finished on primary
+        # client and are not cleared
+        LOGGER.debug("Primary client published 'push'")
+        add_to_ongoing(msg)
+        return True
+    return False
+
+
+def _handle_ack_message(msg):
+    if msg.type == "ack":
+        LOGGER.debug("Primary client finished transfer")
+        _ = add_to_file_cache(msg)
+        _ = clean_ongoing_transfer(get_msg_uid(msg))
+        return True
+    return False
+
+
+def _handle_message_from_another_client(msg):
+    if msg.type == "file" and "request_address" not in msg.data:
+        LOGGER.debug("Ignoring 'file' message from primary client.")
+        add_to_ongoing(msg)
+        _ = add_to_file_cache(msg)
+        _ = clean_ongoing_transfer(get_msg_uid(msg))
+        return True
+    return False
+
+
 def clean_ongoing_transfer(uid):
     """Clear transfer for the given UID from the cache."""
     with ongoing_transfers_lock:
@@ -450,12 +488,15 @@ def make_uris(msg, destination, login=None):
 
 
 def replace_mda(msg, kwargs):
-    """Replace messate metadata with itmes in kwargs dict."""
+    """Replace messate metadata with items in kwargs dict."""
     for key in msg.data:
         if key in kwargs:
-            replacement = dict(item.split(':')
-                               for item in kwargs[key].split('|'))
-            msg.data[key] = replacement[msg.data[key]]
+            try:
+                replacement = dict(item.split(':') for item in kwargs[key].split('|'))
+                replacement = replacement[msg.data[key]]
+            except ValueError:
+                replacement = kwargs[key]
+            msg.data[key] = replacement
     return msg
 
 
@@ -514,12 +555,12 @@ def get_next_msg(uid):
         return ongoing_transfers[uid].pop(0)
 
 
-def add_timer(timeout, callback, msg, *args, **kwargs):
+def add_request_push_timer(timeout, msg, *args, **kwargs):
     """Add a timer for hot spare."""
     huid = get_msg_uid(msg)
     cargs = [msg] + list(args)
     with hot_spare_timer_lock:
-        timer = CTimer(timeout, callback, args=cargs, kwargs=kwargs)
+        timer = CTimer(timeout, request_push, args=cargs, kwargs=kwargs)
         ongoing_hot_spare_timers[huid] = timer
         ongoing_hot_spare_timers[huid].start()
     LOGGER.debug("Added timer for UID %s.", huid)
@@ -565,50 +606,35 @@ def request_push(msg_in, destination, login=None, publisher=None, **kwargs):
         _ = clean_ongoing_transfer(huid)
         return
 
+    _request_files(huid, destination, login, publisher, **kwargs)
+
+
+def _request_files(huid, destination, login, publisher, **kwargs):
     for msg in iterate_messages(huid):
-        try:
-            _destination = compose(destination, msg.data)
-        except KeyError as ke:
-            LOGGER.error("Format identifier is missing from the msg.data: %s", str(ke))
-            raise
-        except ValueError as ve:
-            LOGGER.error("Type of format identifier doesn't match the type in m msg.data: %s", str(ve))
-            raise
-        except AttributeError as ae:
-            LOGGER.error("msg or msg.data is None: %s", str(ae))
-            raise
+        _destination = _compose_destination(destination, msg)
+
         req, fake_req = create_push_req_message(msg, _destination, login)
         LOGGER.info("Requesting: %s", str(fake_req))
-        timeout = float(kwargs["transfer_req_timeout"])
         local_dir = create_local_dir(_destination, kwargs.get('ftp_root', '/'))
 
         publisher.send(str(fake_req))
-        response, hostname = send_request(msg, req, timeout)
+
+        response, hostname = send_request(msg, req, float(kwargs["transfer_req_timeout"]))
 
         if response and response.type in ['file', 'collection', 'dataset']:
             LOGGER.debug("Server done sending file")
             add_to_file_cache(msg)
-            # Send an 'ack' message so that possible hot spares know
-            # the primary has completed the request
-            msg = Message(msg.subject, 'ack', msg.data)
-            LOGGER.debug("Sending a public 'ack' of completed transfer: %s",
-                         str(msg))
-            publisher.send(str(msg))
+            _send_ack_message(msg, publisher)
+
             try:
-                lmsg = unpack_and_create_local_message(response, local_dir,
-                                                       **kwargs)
+                lmsg = unpack_and_create_local_message(response, local_dir, **kwargs)
+                lmsg = _update_local_message(lmsg, _destination, login, response, **kwargs)
             except IOError:
                 LOGGER.exception("Couldn't unpack %s", str(response))
                 continue
-            if publisher:
-                lmsg = make_uris(lmsg, _destination, login)
-                lmsg.data['origin'] = response.data['request_address']
-                lmsg.data.pop('request_address', None)
-                lmsg = replace_mda(lmsg, kwargs)
-                lmsg.data.pop('destination', None)
 
-                LOGGER.debug("publishing %s", str(lmsg))
-                publisher.send(str(lmsg))
+            LOGGER.debug("publishing %s", str(lmsg))
+            publisher.send(str(lmsg))
             terminate_transfers(huid, float(kwargs["req_timeout"]))
             break
         else:
@@ -620,21 +646,58 @@ def request_push(msg_in, destination, login=None, publisher=None, **kwargs):
         terminate_transfers(huid, float(kwargs["req_timeout"]))
 
 
+def _compose_destination(destination, msg):
+    try:
+        _destination = compose(destination, msg.data)
+    except KeyError as ke:
+        LOGGER.error("Format identifier is missing from the msg.data: %s", str(ke))
+        raise
+    except ValueError as ve:
+        LOGGER.error("Type of format identifier doesn't match the type in m msg.data: %s", str(ve))
+        raise
+    except AttributeError as ae:
+        LOGGER.error("msg or msg.data is None: %s", str(ae))
+        raise
+    return _destination
+
+
+def _send_ack_message(msg, publisher):
+    """Send an 'ack' message.
+
+    This is for the possible hot spare clients so they know the primary has completed the request.
+    """
+    msg = Message(msg.subject, 'ack', msg.data)
+    LOGGER.debug("Sending a public 'ack' of completed transfer: %s", str(msg))
+    publisher.send(str(msg))
+
+
+def _update_local_message(lmsg, _destination, login, response, **kwargs):
+    lmsg = make_uris(lmsg, _destination, login)
+    lmsg.data['origin'] = response.data['request_address']
+    lmsg.data.pop('request_address', None)
+    lmsg = replace_mda(lmsg, kwargs)
+    lmsg.data.pop('destination', None)
+
+    return lmsg
+
+
 class Chain(Thread):
     """The Chain class."""
 
-    def __init__(self, name, config, np_=None, publisher=None):
+    def __init__(self, name, config):
         """Init a chain object."""
         super(Chain, self).__init__()
         self._config = config
         self._name = name
-        self._np = np_
-        self.publisher = publisher
+        self._np = None
+        self.publisher = None
         self.listeners = {}
         self.listener_died_event = Event()
         self.running = True
+        self.setup_publisher()
 
-        # Setup publisher if not provided
+    def setup_publisher(self):
+        """Initialize publisher."""
         if self._np is None:
             try:
                 nameservers = self._config["nameservers"]
@@ -648,9 +711,9 @@ class Chain(Thread):
             except (KeyError, NameError):
                 pass
 
-    def setup_listeners(self, callback):
+    def setup_listeners(self, keep_providers=None):
         """Set up the listeners."""
-        self.callback = callback
+        keep_providers = keep_providers or []
         try:
             topics = []
             if "topic" in self._config:
@@ -660,6 +723,9 @@ class Chain(Thread):
                 # Subscribe also to heartbeat messages of other clients
                 topics.append(CLIENT_HEARTBEAT_TOPIC_BASE + '_' + self._name)
             for provider in self._config["providers"]:
+                if provider in keep_providers and provider in self.listeners:
+                    LOGGER.debug("Not restarting Listener to %s, config not changed.", provider)
+                    continue
                 if '/' in provider.split(':')[-1]:
                     parts = urlparse(provider)
                     if parts.scheme != '':
@@ -676,7 +742,6 @@ class Chain(Thread):
                 listener = Listener(
                     provider,
                     topics,
-                    callback,
                     publisher=self.publisher,
                     die_event=self.listener_died_event,
                     **self._config)
@@ -728,6 +793,12 @@ class Chain(Thread):
                 return False
         return True
 
+    def get_unchanged_providers(self, other_config):
+        """Get a list of providers that have not changed between this and other config."""
+        if self._config["topic"] != other_config["topic"]:
+            return []
+        return list(set(self._config["providers"]).intersection(set(other_config["providers"])))
+
     def publisher_needs_restarting(self, other_config):
         """Check that current config is the same as `other_config`."""
         for key in ["nameservers", "publish_port"]:
@@ -735,57 +806,76 @@ class Chain(Thread):
                 return True
         return False
 
-    def reset_listeners(self):
-        """Reset the listeners."""
-        for listener in self.listeners.values():
-            listener.stop()
-        self.listeners = {}
+    def refresh(self, new_config):
+        """Refresh the chain with new config."""
+        publisher_needs_restarting = self.publisher_needs_restarting(new_config)
+        unchanged_providers = self.get_unchanged_providers(new_config)
+        self._config = new_config
+        if publisher_needs_restarting:
+            self._refresh_publisher()
+        self._refresh_listeners(unchanged_providers)
+        if not self.running:
+            self.start()
 
-    def stop(self, stop_publisher=True):
+    def _refresh_publisher(self):
+        self._stop_publisher()
+        self.setup_publisher()
+
+    def _refresh_listeners(self, unchanged_providers):
+        self.reset_listeners(keep_providers=unchanged_providers)
+        self.setup_listeners(keep_providers=unchanged_providers)
+
+    def reset_listeners(self, keep_providers=None):
+        """Reset the listeners."""
+        keep_providers = keep_providers or []
+        kept_listeners = {}
+        for key, listener in self.listeners.items():
+            if key in keep_providers:
+                kept_listeners[key] = listener
+                continue
+            listener.stop()
+        self.listeners = kept_listeners
+
+    def stop(self):
         """Stop the chain."""
+        self._stop_publisher()
         self.running = False
-        if self._np and stop_publisher:
-            self._np.stop()
         self.reset_listeners()
 
+    def _stop_publisher(self):
+        if self._np:
+            self._np.stop()
+            self._np = None
+
     def restart(self):
         """Restart the chain, return a new running instance."""
         self.stop()
         new_chain = self.__class__(self._name, self._config)
-        new_chain.setup_listeners(self.callback)
+        new_chain.setup_listeners()
         new_chain.start()
         return new_chain
 
 
-def reload_config(filename, chains, callback=request_push):
+def reload_config(filename, chains):
     """Rebuild chains if needed (if the configuration changed) from *filename*."""
     LOGGER.debug("New config file detected: %s", filename)
 
     new_configs = read_config(filename)
 
     # setup new chains
-
     for key, new_config in new_configs.items():
-        np_ = None
-        publisher = None
         if key in chains:
             if chains[key].config_equals(new_config):
                 continue
-
-            publisher_needs_restarting = chains[key].publisher_needs_restarting(new_config)
-            if not publisher_needs_restarting:
-                np_ = chains[key]._np
-                publisher = chains[key].publisher
-
-            chains[key].stop(stop_publisher=publisher_needs_restarting)
             verb = "Updated"
             LOGGER.debug("Updating %s", key)
         else:
             verb = "Added"
             LOGGER.debug("Adding %s", key)
-        chains[key] = Chain(key, new_config, np_=np_, publisher=publisher)
-        chains[key].setup_listeners(callback)
-        chains[key].start()
+            chains[key] = Chain(key, new_config)
+            chains[key].start()
+
+        chains[key].refresh(new_config)
 
         LOGGER.debug("%s %s", verb, key)
 
@@ -892,50 +982,10 @@ class PushRequester(object):
 
         return rep
 
-# Generic event handler
-
-
-class EventHandler(pyinotify.ProcessEvent):
-    """Handle events with a generic *fun* function."""
-
-    def __init__(self, fun, *args, **kwargs):
-        """Initialize handler."""
-        pyinotify.ProcessEvent.__init__(self, *args, **kwargs)
-        self._fun = fun
-
-    def process_IN_CLOSE_WRITE(self, event):
-        """Process on closing after writing."""
-        self._fun(event.pathname)
-
-    def process_IN_CREATE(self, event):
-        """Process on closing after linking."""
-        try:
-            if os.stat(event.pathname).st_nlink > 1:
-                self._fun(event.pathname)
-        except OSError:
-            return
-
-    def process_IN_MOVED_TO(self, event):
-        """Process on closing after moving."""
-        self._fun(event.pathname)
-
-
-class StatCollector(object):
-    """StatCollector class."""
-
-    def __init__(self, statfile):
-        """Initialize collector."""
-        self.statfile = statfile
-
-    def collect(self, msg, *args, **kwargs):
-        """Collect."""
-        with open(self.statfile, 'a') as fd:
-            fd.write(time.asctime() + " - " + str(msg) + "\n")
-
 
 def terminate(chains):
     """Terminate client chains."""
-    for chain in six.itervalues(chains):
+    for chain in chains.values():
         chain.stop()
     LOGGER.info("Shutting down.")
     print("Thank you for using pytroll/move_it_client."
